import * as TaskManager from 'expo-task-manager';
import * as Notifications from 'expo-notifications';
import { Platform } from 'react-native';
import { API_URL, APP_VERSION } from 'src/constants';
import axios from 'axios';
import { API } from 'src/types';
import { storage, StoreType } from 'src/storage';

const BACKGROUND_NOTIFICATION_TASK = 'BACKGROUND-NOTIFICATION-TASK';

TaskManager.defineTask(
  BACKGROUND_NOTIFICATION_TASK,
  async ({ data, error }: { data?: any; error?: any }) => {
    console.log('Background notification task started', data);
    if (error) {
      return;
    }

    if (data) {
      const token = storage.get('token', StoreType.STRING);
      let groupToken;
      let messageId;

      if (Platform.OS === 'ios') {
        groupToken = data?.UIApplicationLaunchOptionsRemoteNotificationKey?.group_token;
        messageId = data?.UIApplicationLaunchOptionsRemoteNotificationKey?.message_id;
      } else {
        groupToken = data?.notification?.data?.group_token;
        messageId = data?.notification?.data?.message_id;
      }

      if (!groupToken || !messageId) return;

      try {
        await axios.postForm(
          API_URL + '/' + API.GROUP_MESSAGES_RECEIVED,
          {
            token,
            group_token: groupToken,
            messages_id: [messageId]
          },
          {
            headers: {
              Platform: Platform.OS,
              'App-Version': APP_VERSION
            }
          }
        );
      } catch (err) {
        console.error('Error sending notification data:', err);
      }
    }
  }
);

export const registerBackgroundNotificationTask = async () => {
  const isRegistered = await TaskManager.isTaskRegisteredAsync(BACKGROUND_NOTIFICATION_TASK);

  if (!isRegistered) {
    await Notifications.registerTaskAsync(BACKGROUND_NOTIFICATION_TASK);
  }
};

export const unregisterBackgroundNotificationTask = async () => {
  const isRegistered = await TaskManager.isTaskRegisteredAsync(BACKGROUND_NOTIFICATION_TASK);

  if (isRegistered) {
    await TaskManager.unregisterTaskAsync(BACKGROUND_NOTIFICATION_TASK);
  }
};
